@@ -332,12 +332,7 @@ fn main() {
   }
   let mut y = StaticVec::<MyStruct, 2>::new();
   y.push(MyStruct { s: "hey" });
-  let z: &MyStruct = y.as_ref();
-  println!("{}", z.s);
-  let zz: &mut MyStruct = y.as_mut();
-  println!("{}", zz.s);
-  let zzz: &[MyStruct] = y.as_ref();
-  println!("{}", zzz[0].s);
-  let zzzz: &mut [MyStruct] = y.as_mut();
-  println!("{}", zzzz[0].s);
+  for x in y.as_mut() {
+    println!("{}", x.s);
+  }
 }
