@@ -9,6 +9,7 @@
 #![feature(exact_size_is_empty)]
 #![feature(trusted_len)]
 #![feature(slice_partition_dedup)]
+#![feature(slice_concat_trait)]
 
 //Literally just for stable-sort.
 #[cfg(any(feature = "std", rustdoc))]
@@ -25,11 +26,12 @@ pub use crate::trait_impls::*;
 use crate::utils::*;
 use core::cmp::{Ord, PartialEq};
 use core::marker::PhantomData;
-use core::mem::MaybeUninit;
+use core::mem::{self, MaybeUninit};
 use core::ops::{Bound::Excluded, Bound::Included, Bound::Unbounded, RangeBounds};
 use core::ptr;
 
 mod iterators;
+#[macro_use]
 mod macros;
 mod trait_impls;
 #[doc(hidden)]
@@ -46,12 +48,12 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   ///Returns a new StaticVec instance.
   #[inline(always)]
   pub fn new() -> Self {
-    unsafe {
-      Self {
-        //Sound because data is an array of MaybeUninit<T>, not an array of T.
-        data: MaybeUninit::uninit().assume_init(),
-        length: 0,
-      }
+    //TODO: Open issue asking them to make `assume_init` a const fn, which it can be,
+    //because it just calls an intrinsic and then another const fn. Then we could have const `new`!
+    Self {
+      //Sound because data is an array of MaybeUninit<T>, not an array of T.
+      data: unsafe { MaybeUninit::uninit().assume_init() },
+      length: 0,
     }
   }
 
@@ -62,17 +64,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   #[inline]
   pub fn new_from_slice(values: &[T]) -> Self
   where T: Copy {
-    unsafe {
-      let mut data_: [MaybeUninit<T>; N] = MaybeUninit::uninit().assume_init();
-      let fill_length = values.len().min(N);
-      values
-        .as_ptr()
-        .copy_to_nonoverlapping(data_.as_mut_ptr() as *mut T, fill_length);
-      Self {
-        data: data_,
-        length: fill_length,
-      }
-    }
+    new_from_slice_internal!(values, values.len().min(N))
   }
 
   ///Returns a new StaticVec instance filled with the contents, if any, of a mutable slice reference.
@@ -82,38 +74,16 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   #[inline]
   pub fn new_from_mut_slice(values: &mut [T]) -> Self
   where T: Copy {
-    unsafe {
-      let mut data_: [MaybeUninit<T>; N] = MaybeUninit::uninit().assume_init();
-      let fill_length = values.len().min(N);
-      values
-        .as_ptr()
-        .copy_to_nonoverlapping(data_.as_mut_ptr() as *mut T, fill_length);
-      Self {
-        data: data_,
-        length: fill_length,
-      }
-    }
+    new_from_slice_internal!(values, values.len().min(N))
   }
 
   ///Returns a new StaticVec instance filled with the contents, if any, of an array.
   ///If the array has a length greater than the StaticVec's declared capacity,
   ///any contents after that point are ignored.
   ///The `N2` parameter does not need to be provided explicitly, and can be inferred from the array itself.
-  ///Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues.
   #[inline]
-  pub fn new_from_array<const N2: usize>(values: [T; N2]) -> Self
-  where T: Copy {
-    unsafe {
-      let mut data_: [MaybeUninit<T>; N] = MaybeUninit::uninit().assume_init();
-      let fill_length = N2.min(N);
-      values
-        .as_ptr()
-        .copy_to_nonoverlapping(data_.as_mut_ptr() as *mut T, fill_length);
-      Self {
-        data: data_,
-        length: fill_length,
-      }
-    }
+  pub fn new_from_array<const N2: usize>(mut values: [T; N2]) -> Self {
+    new_from_array_internal!(values)
   }
 
   ///Returns a new StaticVec instance filled with the return value of an initializer function.
@@ -135,6 +105,12 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   pub fn filled_with<F>(mut initializer: F) -> Self
   where F: FnMut() -> T {
     let mut res = Self::new();
+    //You might think it would make more sense to use `push_unchecked` here.
+    //Originally, I did also! However, as of today (November 19, 2019), doing so
+    //both in this function and several others throughout the crate defeats the ability
+    //of `rustc` to fully unroll and autovectorize the constant-bounds loop. If this changes
+    //in the future, feel free to open a PR switching out the manual code for `get_unchecked`, if
+    //you happen to notice it before I do.
     for i in 0..N {
       unsafe {
         res.data.get_unchecked_mut(i).write(initializer());
@@ -261,7 +237,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
       }
       Ok(())
     } else {
-      Err("No space left!")
+      Err("Insufficient remaining capacity!")
     }
   }
 
@@ -544,7 +520,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   pub fn try_extend_from_slice(&mut self, other: &[T]) -> Result<(), &'static str>
   where T: Copy {
     if self.remaining_capacity() < other.len() {
-      Err("No space left!")
+      Err("Insufficient remaining capacity!")
     } else {
       self.extend_from_slice(other);
       Ok(())
