@@ -4,7 +4,7 @@ use core::cmp::{Ordering, PartialOrd};
 #[inline(always)]
 pub(crate) fn distance_between<T>(self_: *const T, origin: *const T) -> usize {
   unsafe {
-    core::intrinsics::panic_if_uninhabited::<T>();
+    assert!(0 < core::mem::size_of::<T>());
     core::intrinsics::exact_div(
       (self_ as usize).wrapping_sub(origin as usize),
       core::intrinsics::size_of::<T>(),
