@@ -3,7 +3,7 @@
 use staticvec::*;
 
 #[cfg(feature = "std")]
-use std::io::{IoSlice, Read, Write};
+use std::io::{IoSlice, IoSliceMut, Read, Write};
 
 #[test]
 fn as_mut_ptr() {
@@ -327,8 +327,37 @@ fn read() {
   assert_eq!(ints.read(&mut buffer2).unwrap(), 0);
   assert_eq!(buffer2, []);
   let mut buffer3 = staticvec![0; 9];
-  assert_eq!(ints.read(buffer3.as_mut_slice()).unwrap(), 9);
-  assert_eq!(ints, buffer3);
+  assert_eq!(ints.read(buffer3.as_mut_slice()).unwrap(), 5);
+  assert_eq!(ints, []);
+  assert_eq!(ints.read(staticvec![].as_mut_slice()).unwrap(), 0);
+}
+
+#[cfg(feature = "std")]
+#[test]
+fn read_exact() {
+  let mut ints = staticvec![1, 2, 3, 4, 6, 7, 8, 9, 10];
+  let mut buffer = [0, 0, 0, 0];
+  ints.read_exact(&mut buffer).unwrap();
+  assert_eq!(buffer, [1, 2, 3, 4]);
+}
+
+#[cfg(feature = "std")]
+#[test]
+fn read_vectored() {
+  let mut ints = staticvec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12];
+  let mut buf1 = [0; 4];
+  let mut buf2 = [0; 4];
+  let mut buf3 = [0; 4];
+  let bufs = &mut [
+    IoSliceMut::new(&mut buf1),
+    IoSliceMut::new(&mut buf2),
+    IoSliceMut::new(&mut buf3),
+  ];
+  assert_eq!(ints.read_vectored(bufs).unwrap(), 12);
+  assert_eq!(
+    "[[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12]]",
+    format!("{:?}", bufs)
+  );
 }
 
 #[test]
@@ -456,7 +485,10 @@ fn truncate() {
 #[test]
 fn try_extend_from_slice() {
   let mut v = StaticVec::<i32, 3>::from([1, 2, 3]);
-  assert_eq!(v.try_extend_from_slice(&[2, 3]), Err("No space left!"));
+  assert_eq!(
+    v.try_extend_from_slice(&[2, 3]),
+    Err("Insufficient remaining capacity!")
+  );
   let mut w = StaticVec::<i32, 4>::from([1, 2, 3]);
   assert_eq!(w.try_extend_from_slice(&[2]), Ok(()));
 }
@@ -477,7 +509,7 @@ fn try_insert() {
 #[test]
 fn try_push() {
   let mut vec = staticvec![1, 2, 3, 4, 5];
-  assert_eq!(vec.try_push(2), Err("No space left!"));
+  assert_eq!(vec.try_push(2), Err("Insufficient remaining capacity!"));
   let mut vec2 = StaticVec::<i32, 4>::new_from_slice(&[1, 2, 3]);
   vec2.push(3);
   assert_eq!(vec2, [1, 2, 3, 3]);
@@ -495,6 +527,15 @@ fn write() {
   assert_eq!(&v[..], &[1, 2, 3, 9, 9, 9, 9, 9]);
 }
 
+#[cfg(feature = "std")]
+#[test]
+fn write_all() {
+  let mut v = StaticVec::<u8, 6>::new();
+  assert!(v.write_all(&[1, 2, 3, 4, 5, 6, 7, 8]).is_err());
+  v.clear();
+  assert!(v.write_all(&[1, 2, 3, 4, 5, 6]).is_ok());
+}
+
 #[cfg(feature = "std")]
 #[test]
 fn write_vectored() {
